"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const pg_1 = require("pg");
const QueryStream = require("pg-query-stream");
const url = require("url");
class PgRx {
    /**
     * Initalize a database connection.
     *
     * It can be initialized with a database url like
     * ```
     * pg://user:password@host:port/database
     * ```
     *
     * or a crednetial object:
     *
     * ``` json
     * {
     *   "user": "postgres",
     *   "database": "tester",
     *   "password": "",
     *   "host": "localhost",
     *   "port": 5432
     * }
     * ```
     *
     * @param config PostgreSQL database connection string or config object
     */
    constructor(config) {
        let connectParams;
        if (typeof config === "string") {
            const params = url.parse(config);
            const auth = params.auth ? params.auth.split(":") : [];
            connectParams = {
                user: auth[0],
                password: auth[1],
                host: params.hostname,
                port: parseInt(params.port, 10),
                database: params.pathname.split("/")[1]
            };
        }
        else {
            connectParams = config;
        }
        this._db = new pg_1.Pool(connectParams);
    }
    /**
     * Close the current database connection.
     */
    end() {
        this._db.end();
    }
    /**
     * Run a query with optional values.
     *
     * This function supports the query formatting of [pg](https://github.com/brianc/node-postgres/wiki/Client#parameterized-queries) and you can construct the query like
     *
     * ``` javascript
     * pgrx.query('SELECT id FROM user WHERE name = $1::text', ['Tom']);
     * ```
     *
     * It will return an observable that emits every row of the query result.
     *
     * @param  sql    Query SQL
     * @param  values Optional query parameters
     * @return        Observable
     */
    query(sql, values) {
        if (!sql || typeof sql !== "string") {
            throw new Error("Invalid queary: " + sql);
        }
        const valuesArray = values
            ? Array.isArray(values)
                ? values
                : [values]
            : undefined;
        const observable = rxjs_1.defer(() => this._db.connect());
        const doQuery = operators_1.concatMap((client) => {
            const queryFn = client.query.bind(client);
            return this._streamQuery(queryFn, sql, valuesArray, client.release.bind(client));
        });
        return observable.pipe(doQuery);
    }
    /**
     * Run queries within a transaction.
     *
     * The callback function receives an object that has a `query()` function to run queries within the transaction and return an observable. To pass the data to the following operator, return an observable in the callback function.
     *
     * ``` javascript
     * pgrx.tx((t) => {
     *  const insert1 = t.query('INSERT INTO user (name) VALUES ($1::text) RETURNING id;', ['Tom']);
     *  const insert2 = t.query('INSERT INTO user (name) VALUES ($1::text) RETURNING id;', ['Joe']);
     *
     *  return insert1.concat(insert2);
     * })
     * .subscribe((row) => console.log(row));
     * ```
     *
     * No data will be emitted if any query in a transaction fails.
     *
     * @param  fn A callback function that returns an observable for database operation.
     * @return    Observable
     */
    tx(fn) {
        const observable = rxjs_1.defer(() => this._db.connect());
        const doQuery = operators_1.concatMap((client) => {
            const queryFn = client.query.bind(client);
            const begin = rxjs_1.defer(() => this._streamQuery(queryFn, "BEGIN;"));
            const commit = rxjs_1.defer(() => this._streamQuery(queryFn, "COMMIT;"));
            const query = fn({
                query: (sql, values) => rxjs_1.defer(() => this._streamQuery(queryFn, sql, values))
            });
            const expanseArray = operators_1.concatMap((results) => rxjs_1.of(...results));
            const handleError = operators_1.catchError(err => {
                return rxjs_1.defer(() => queryFn("ROLLBACK;").then(() => {
                    throw err;
                }));
            });
            const release = operators_1.finalize(() => client.release());
            const execution = rxjs_1.concat(begin, query, commit);
            return execution.pipe(operators_1.toArray(), expanseArray, handleError, release);
        });
        return observable.pipe(doQuery);
    }
    _streamQuery(queryFn, sql, values, cleanup) {
        return rxjs_1.Observable.create(observer => {
            const query = new QueryStream(sql, values);
            const stream = queryFn(query);
            stream.on("data", row => observer.next(row));
            stream.on("error", error => observer.error(error));
            stream.on("end", () => observer.complete());
            return cleanup;
        });
    }
}
exports.default = PgRx;
